@@ -197,13 +197,13 @@ SOFIAPUBFUN issize_t msg_headers_prepare(msg_t *,
 
 #ifdef SU_HAVE_INLINE
 /** Clear encoded data from header structure. */
-static inline void msg_fragment_clear(msg_common_t *h) 
+su_inline void msg_fragment_clear(msg_common_t *h) 
 {
   h->h_data = NULL, h->h_len = 0;
 }
 
 /** Pointer to header parameters. */
-static inline 
+su_inline 
 msg_param_t **msg_header_params(msg_common_t const *h)
 {
   if (h && h->h_class->hc_params) {
@@ -234,7 +234,8 @@ SOFIAPUBFUN int msg_header_replace_item(su_home_t *, msg_common_t *h,
 SOFIAPUBFUN int msg_header_remove_item(msg_common_t *h, char const *name);
 
 /** Append a list of constant items to a list. */
-SOFIAPUBFUN int msg_list_append_items(su_home_t *home, msg_list_t *k,
+SOFIAPUBFUN int msg_list_append_items(su_home_t *home,
+				      msg_list_t *k,
 				      msg_param_t const items[]);
 
 /** Replace a list of constant items on a list */
@@ -242,6 +243,11 @@ SOFIAPUBFUN int msg_list_replace_items(su_home_t *home,
 				       msg_list_t *k,
 				       msg_param_t const items[]);
 
+SOFIAPUBFUN int msg_header_join_items(su_home_t *home,
+				      msg_common_t *dst,
+				      msg_common_t const *src,
+				      int duplicate);
+
 SOFIAPUBFUN issize_t msg_random_token(char token[], isize_t tlen,
 				      void const *d, isize_t dlen);
 
