@@ -35,7 +35,7 @@
  * @date Created: Wed Mar  8 11:38:18 EET 2006  ppessi
  */
 
-/** NUA preferences.
+/**@internal @brief NUA preferences. 
  *
  * This structure contains values for various preferences and a separate
  * bitmap (nhp_set) for each preference. Preferences are set using
@@ -95,9 +95,11 @@ typedef struct nua_handle_preferences
   unsigned         nhp_service_route_enable:1;
   /** Enable Path */
   unsigned         nhp_path_enable:1;
+  /** Authentication cache policy */
+  unsigned         nhp_auth_cache:1;
+
   /** Always include id with Event: refer */
   unsigned         nhp_refer_with_id:1;
-
   unsigned:0;
 
   /* Default lifetime for implicit subscriptions created by REFER */
@@ -106,7 +108,7 @@ typedef struct nua_handle_preferences
   /* Subscriber state, i.e. nua_substate_pending */
   unsigned         nhp_substate;
 
-  /* REGISTER Keepalive intervals */
+  /* REGISTER keepalive intervals */
   unsigned         nhp_keepalive, nhp_keepalive_stream;
   char const      *nhp_registrar;
 
@@ -122,14 +124,19 @@ typedef struct nua_handle_preferences
   char const         *nhp_m_features;
   char const         *nhp_instance;
 
-  /**< Outbound OPTIONS */
+  /** Outbound OPTIONS */
   char const         *nhp_outbound; 
   
-  /**< Network detection: NONE, INFORMAL, TRY_FULL */
+  /** Network detection: NONE, INFORMAL, TRY_FULL */
   int                 nhp_detect_network_updates;
+  
+  sip_allow_t        *nhp_appl_method;
 
-  /* A bit for each feature set by application */
-  struct {
+  /** Initial route set */
+  sip_route_t        *nhp_initial_route;
+
+  union { struct {
+    /* A bit for each feature set by application */
     unsigned nhb_retry_count:1;
     unsigned nhb_max_subscriptions:1;
 
@@ -155,6 +162,7 @@ typedef struct nua_handle_preferences
     unsigned nhb_media_features:1;
     unsigned nhb_service_route_enable:1;
     unsigned nhb_path_enable:1;
+    unsigned nhb_auth_cache:1;
     unsigned nhb_refer_with_id:1;
     unsigned nhb_refer_expires:1;
     unsigned nhb_substate:1;
@@ -164,8 +172,8 @@ typedef struct nua_handle_preferences
 
     unsigned nhb_allow:1;
     unsigned nhb_supported:1;
-    unsigned nhb_allow_events:1;
     unsigned :0;		/* at most 32 bits ... */
+    unsigned nhb_allow_events:1;
     unsigned nhb_user_agent:1;
     unsigned nhb_organization:1;
 
@@ -176,10 +184,16 @@ typedef struct nua_handle_preferences
     unsigned nhb_instance:1;
     unsigned nhb_outbound:1;
     unsigned nhb_detect_network_updates:1;
+    unsigned nhb_appl_method:1;
+    unsigned nhb_initial_route:1;
     unsigned :0;
-  } nhp_set;
+  } set_bits; 
+    unsigned set_unsigned[2];
+  } nhp_set_;
 } nua_handle_preferences_t;
 
+#define nhp_set nhp_set_.set_bits
+
 #define DNHP_GET(dnhp, pref) ((dnhp)->nhp_##pref)
 
 #define NHP_GET(nhp, dnhp, pref)					\
@@ -216,4 +230,9 @@ typedef struct nua_handle_preferences
   (NHP_ISSET((nh)->nh_prefs, pref) &&					\
    (nh)->nh_nua->nua_dhandle->nh_prefs != (nh)->nh_prefs)
 
+/* Check if preference has been set by applicationx */
+#define NUA_PISSET(nua, nh, pref)					\
+  (NHP_ISSET((nua)->nua_dhandle->nh_prefs, pref) ||			\
+   ((nh) && NHP_ISSET((nh)->nh_prefs, pref)))
+
 #endif /* NUA_PARAMS_H */
