@@ -62,7 +62,7 @@
 
 #ifndef NUA_DIALOG_H
 #define NUA_OWNER_T struct nua_handle_s
-#include <nua_dialog.h>
+#include "nua_dialog.h"
 #endif
 
 SOFIA_BEGIN_DECLS
@@ -72,7 +72,7 @@ SOFIA_BEGIN_DECLS
 #endif
 
 #ifndef NUA_PARAMS_H
-#include <nua_params.h>
+#include "nua_params.h"
 #endif
 
 typedef struct event_s event_t;
@@ -129,7 +129,7 @@ su_inline int nua_handle_unref_by(nua_handle_t *nh, char const *by)
 
 #endif
 
-/** NUA handle. 
+/** @internal @brief NUA handle. 
  *
  */
 struct nua_handle_s 
@@ -197,6 +197,8 @@ int nh_is_special(nua_handle_t *nh)
   return nh == NULL || nh->nh_special;
 }
 
+typedef struct nua_event_frame_s nua_event_frame_t;
+
 extern char const nua_internal_error[];
 
 #define NUA_INTERNAL_ERROR 900, nua_internal_error
@@ -214,12 +216,14 @@ struct nua_s {
   nua_callback_f       nua_callback;
   nua_magic_t         *nua_magic;
 
-  nua_saved_event_t    nua_current[1];
+  nua_event_frame_t   *nua_current;
   nua_saved_event_t    nua_signal[1];
 
   /* Engine state flags */
   unsigned             nua_shutdown_started:1; /**< Shutdown initiated */
   unsigned             nua_shutdown_final:1; /**< Shutdown is complete */
+
+  unsigned             nua_from_is_set;
   unsigned :0;
   
   /**< Used by stop-and-wait args calls */
@@ -307,7 +311,8 @@ nua_stack_signal_handler
   nua_stack_set_params, nua_stack_get_params,
   nua_stack_register, 
   nua_stack_invite, nua_stack_ack, nua_stack_cancel, 
-  nua_stack_bye, nua_stack_info, nua_stack_update, 
+  nua_stack_bye, nua_stack_info, nua_stack_update,
+  nua_stack_prack,
   nua_stack_options, nua_stack_publish, nua_stack_message, 
   nua_stack_subscribe, nua_stack_notify, nua_stack_refer,
   nua_stack_method;
@@ -336,8 +341,7 @@ nua_handle_t *nua_stack_incoming_handle(nua_t *nua,
 					sip_t const *sip,
 					int create_dialog);
 
-int nua_stack_init_handle(nua_t *nua, nua_handle_t *nh, 
-			  tag_type_t tag, tag_value_t value, ...);
+int nua_stack_init_handle(nua_t *nua, nua_handle_t *nh, tagi_t const *tags);
 
 enum nh_kind {
   nh_has_nothing,
