@@ -36,7 +36,7 @@
 
 #define NTA_OUTGOING_MAGIC_T struct outbound
 
-#include <outbound.h>
+#include "outbound.h"
 
 #include <sofia-sip/hostdomain.h>
 #include <sofia-sip/sip.h>
@@ -581,7 +581,10 @@ int outbound_gruuize(outbound_t *ob, sip_t const *sip)
     return 0;
   }
 
-  gruu = (char *)msg_header_find_param(m->m_common, "gruu=");
+  gruu = (char *)msg_header_find_param(m->m_common, "pub-gruu=");
+  
+  if (gruu == NULL || gruu[0] == '\0')
+    gruu = (char *)msg_header_find_param(m->m_common, "gruu=");
 
   if (gruu == NULL || gruu[0] == '\0')
     return 0;
@@ -860,11 +863,20 @@ static int response_to_keepalive_options(outbound_t *ob,
     loglevel = 1;		/* XXX ... for now */
 
     if (loglevel >= SU_LOG->log_level) {
-      su_llog(SU_LOG, loglevel,       
-	      "outbound(%p): %s <" URL_PRINT_FORMAT ">\n",
-	      (void *)ob->ob_owner,
-	      failed ? "FAILED to validate" : "validated",
-	      URL_PRINT_ARGS(ob->ob_rcontact->m_url));
+      sip_contact_t const *m = ob->ob_rcontact;
+
+      if  (m)
+	su_llog(SU_LOG, loglevel,       
+		"outbound(%p): %s <" URL_PRINT_FORMAT ">\n",
+		(void *)ob->ob_owner,
+		failed ? "FAILED to validate" : "validated",
+		URL_PRINT_ARGS(m->m_url));
+      else
+	su_llog(SU_LOG, loglevel,       
+		"outbound(%p): %s registration\n",
+		(void *)ob->ob_owner,
+		failed ? "FAILED to validate" : "validated");
+
       if (failed)
 	su_llog(SU_LOG, loglevel, "outbound(%p): FAILED with %u %s\n", 
 		(void *)ob->ob_owner, status, phrase);
@@ -1004,14 +1016,14 @@ int outbound_contacts_from_via(outbound_t *ob, sip_via_t const *via)
 
   v = v0; *v0 = *via; v0->v_next = NULL;
 
-  dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1,
-				   NULL, v, v->v_protocol, NULL);
+  dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1, 
+				   v, v->v_protocol, NULL);
 
   if (ob->ob_instance && ob->ob_reg_id != 0)
     snprintf(reg_id_param, sizeof reg_id_param, ";reg-id=%u", ob->ob_reg_id);
 
   rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 0,
-				   NULL, v, v->v_protocol, 
+				   v, v->v_protocol, 
 				   ob->ob_instance, reg_id_param, NULL);
     
   v = sip_via_dup(home, v);
@@ -1098,6 +1110,8 @@ int outbound_set_contact(outbound_t *ob,
       previous = ob->ob_rcontact;
   }
   else if (application_contact) {
+    rcontact = sip_contact_dup(home, application_contact);
+
     if (!ob->ob_rcontact || 
 	url_cmp_all(ob->ob_rcontact->m_url, application_contact->m_url)) {
       contact_uri_changed = 1;
@@ -1112,7 +1126,7 @@ int outbound_set_contact(outbound_t *ob,
     char reg_id_param[20];
 
     dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1, 
-				     NULL, v, tport, NULL);
+				     v, tport, NULL);
     if (!dcontact)
       return -1;
 
@@ -1120,7 +1134,7 @@ int outbound_set_contact(outbound_t *ob,
       snprintf(reg_id_param, sizeof reg_id_param, ";reg-id=%u", ob->ob_reg_id);
 
     rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 0,
-				     NULL, v, v->v_protocol, 
+				     v, v->v_protocol, 
 				     ob->ob_instance, reg_id_param, NULL);
     if (!rcontact)
       return -1;
