@@ -34,6 +34,7 @@
 #include "config.h"
 
 #include "test_nua.h"
+#include "sofia-sip/tport_tag.h"
 
 #if HAVE_FUNC
 #elif HAVE_FUNCTION
@@ -71,9 +72,9 @@ int test_nua_api_errors(struct context *ctx)
   TEST(status, 200); TEST_S(phrase, sip_200_OK);
 
   su_log_init(nua_log);
+  level = nua_log->log_level;
   if (!(tstflags & tst_verbatim))
     su_log_set_level(nua_log, 0);  /* Log at level 0 by default */
-  level = nua_log->log_level;
 
   TEST_1(!nua_create(NULL, NULL, NULL, TAG_END()));
   TEST_VOID(nua_shutdown(NULL));
@@ -142,7 +143,62 @@ int test_nua_api_errors(struct context *ctx)
 
 /* ======================================================================== */
 
-int test_stack_errors(struct context *ctx)
+void destroy_callback(nua_event_t event,
+		      int status, char const *phrase,
+		      nua_t *nua, struct context *ctx,
+		      nua_handle_t *nh, struct call *call,
+		      sip_t const *sip,
+		      tagi_t tags[])
+{
+  if (status >= 200) {
+    nua_destroy(ctx->a.nua), ctx->a.nua = NULL;
+    su_root_break(ctx->root);
+  }
+}
+
+/* Test different nua_destroy() corner cases */
+int test_nua_destroy(struct context *ctx)
+{
+  BEGIN();
+  
+  struct endpoint *a = &ctx->a;
+
+  TEST_1(ctx->root = su_root_create(NULL));
+
+#if 0
+  a->nua = nua_create(ctx->root, destroy_callback, ctx,
+		      NUTAG_URL("sip:0.0.0.0:*"),
+		      TAG_IF(ctx->a.logging, TPTAG_LOG(1)),
+		      TAG_END());
+  TEST_1(a->nua);
+
+  nua_get_params(a->nua, TAG_ANY(), TAG_END());
+
+  su_root_run(ctx->root);
+
+  TEST_1(a->nua == NULL);
+#endif
+
+  a->nua = nua_create(ctx->root, destroy_callback, ctx,
+		      NUTAG_URL("sip:0.0.0.0:*"),
+		      TAG_IF(ctx->a.logging, TPTAG_LOG(1)),
+		      TAG_END());
+  TEST_1(a->nua);
+
+  nua_shutdown(a->nua);
+
+  su_root_run(ctx->root);
+
+  TEST_1(a->nua == NULL);
+
+  su_root_destroy(ctx->root), ctx->root = NULL;
+
+  END();
+}
+
+/* ======================================================================== */
+
+int test_byecancel_without_invite(struct context *ctx)
 {
   BEGIN();
 
@@ -152,9 +208,6 @@ int test_stack_errors(struct context *ctx)
 
   int internal_error = 900;
 
-  if (print_headings)
-    printf("TEST NUA-1.2: Stack error handling\n");
-
   if (print_headings)
     printf("TEST NUA-1.2.1: CANCEL without INVITE\n");
 
@@ -179,7 +232,8 @@ int test_stack_errors(struct context *ctx)
   if (print_headings)
     printf("TEST NUA-1.2.2: BYE without INVITE\n");
 
-  TEST_1(a_call->nh = nua_handle(a->nua, a_call, SIPTAG_TO(b->to), TAG_END()));
+  a_call->nh = nua_handle(a->nua, a_call, SIPTAG_TO(b->to), TAG_END());
+  TEST_1(a_call->nh);
 
   BYE(a, a_call, a_call->nh, TAG_END());
 
@@ -195,15 +249,25 @@ int test_stack_errors(struct context *ctx)
   if (print_headings)
     printf("TEST NUA-1.2.2: PASSED\n");
 
-  if (!ctx->proxy_tests)
-    goto nua_1_2_5;
+  END();
+}
+
+
+int test_unregister_without_register(struct context *ctx)
+{
+  BEGIN();
+
+  struct endpoint *a = &ctx->a, *b = &ctx->b;
+  struct call *a_call = a->call;
+  struct event *e;
 
   /* -Un-register without REGISTER--------------------------------------- */
 
   if (print_headings)
     printf("TEST NUA-1.2.3: unregister without register\n");
 
-  TEST_1(a_call->nh = nua_handle(a->nua, a_call, SIPTAG_TO(a->to), TAG_END()));
+  a_call->nh = nua_handle(a->nua, a_call, SIPTAG_TO(a->to), TAG_END());
+  TEST_1(a_call->nh);
 
   UNREGISTER(a, a_call, a_call->nh, TAG_END());
 
@@ -227,7 +291,8 @@ int test_stack_errors(struct context *ctx)
   if (print_headings)
     printf("TEST NUA-1.2.4: unpublish without publish\n");
 
-  TEST_1(a_call->nh = nua_handle(a->nua, a_call, SIPTAG_TO(b->to), TAG_END()));
+  a_call->nh = nua_handle(a->nua, a_call, SIPTAG_TO(b->to), TAG_END());
+  TEST_1(a_call->nh);
 
   UNPUBLISH(a, a_call, a_call->nh, TAG_END());
 
@@ -242,10 +307,22 @@ int test_stack_errors(struct context *ctx)
 
   if (print_headings)
     printf("TEST NUA-1.2.4: PASSED\n");
+  
+  END();
+}
 
-  /* -terminate without notifier--------------------------------------- */
+/* -terminate without notifier--------------------------------------- */
+
+int test_terminate_without_notifier(struct context *ctx)
+{
+  BEGIN();
+
+  struct endpoint *a = &ctx->a, *b = &ctx->b;
+  struct call *a_call = a->call;
+  struct event *e;
+
+  int internal_error = 900;
 
- nua_1_2_5:
   if (print_headings)
     printf("TEST NUA-1.2.5: terminate without notifier\n");
 
@@ -276,8 +353,108 @@ int test_stack_errors(struct context *ctx)
   if (print_headings)
     printf("TEST NUA-1.2.5: PASSED\n");
 
+  END();
+}
+
+int destroy_on_503(CONDITION_PARAMS)
+{
+  save_event_in_list(ctx, event, ep, call);
+
+  if (status == 503) {
+    assert(nh == call->nh);
+    nua_handle_destroy(call->nh), call->nh = NULL;
+  }
+
+  return
+    nua_r_set_params <= event && event < nua_i_network_changed
+    && status >= 200;
+}
+
+
+int test_register_503(struct context *ctx)
+{
+  BEGIN();
+
+  struct endpoint *a = &ctx->a;
+  struct call *a_reg = a->reg;
+  struct event *e;
+
+/* REGISTER test
+
+   A
+   |------REGISTER--\
+   |<-------503-----/
+   |			
+
+*/
+
+  if (print_headings)
+    printf("TEST NUA-1.2.6: REGISTER with bad domain\n");
+
+  TEST_1(a_reg->nh = nua_handle(a->nua, a_reg, TAG_END()));
+
+  REGISTER(a, a_reg, a_reg->nh, 
+	   NUTAG_REGISTRAR(URL_STRING_MAKE("sip:bad.domain")),
+	   SIPTAG_TO_STR("sip:lissu@bad.domain"),
+	   TAG_END());
+  run_a_until(ctx, -1, destroy_on_503);
+
+  TEST_1(e = a->events->head);
+  TEST_E(e->data->e_event, nua_r_register);
+  TEST(e->data->e_status, 503);
+  TEST_1(!e->next);
+  free_events_in_list(ctx, a->events);
+
+  TEST_1(a_reg->nh = nua_handle(a->nua, a_reg, TAG_END()));
+
+  REGISTER(a, a_reg, a_reg->nh, 
+	   NUTAG_REGISTRAR(URL_STRING_MAKE("sip:bad.domain")),
+	   SIPTAG_TO_STR("sip:lissu@bad.domain"),
+	   TAG_END());
+  nua_handle_destroy(a_reg->nh), a_reg->nh = NULL;
+
+  if (print_headings)
+    printf("TEST NUA-1.2.6: PASSED\n");
+
+  END();
+}
+
+int test_stack_errors(struct context *ctx)
+{
+  BEGIN();
+
+  struct endpoint *a = &ctx->a;
+
+  if (print_headings)
+    printf("TEST NUA-1.2: Stack error handling\n");
+
+  TEST_1(ctx->root == NULL);
+  TEST_1(ctx->root = su_root_create(NULL));
+
+  a->nua = nua_create(ctx->root, a_callback, ctx,
+		      NUTAG_URL("sip:0.0.0.0:*"),
+		      TAG_IF(ctx->a.logging, TPTAG_LOG(1)),
+		      TAG_END());
+  TEST_1(a->nua);
+
+  TEST(test_byecancel_without_invite(ctx), 0);
+  if (ctx->proxy_tests)
+    TEST(test_unregister_without_register(ctx), 0);
+  TEST(test_terminate_without_notifier(ctx), 0);
+  TEST(test_register_503(ctx), 0);
+  TEST(test_register_503(ctx), 0);
+
+  nua_shutdown(a->nua);
+
+  run_a_until(ctx, -1, until_final_response);
+  
+  TEST_VOID(nua_destroy(a->nua));
+
+  su_root_destroy(ctx->root), ctx->root = NULL;
+	 
   if (print_headings)
     printf("TEST NUA-1.2: PASSED\n");
 
   END();
 }
+
