@@ -43,6 +43,8 @@
 #define __func__ "test_nua_params"
 #endif
 
+sip_route_t *GLOBAL_ROUTE;
+
 int test_tag_filter(void)
 {
   BEGIN();
@@ -104,7 +106,6 @@ int test_nua_params(struct context *ctx)
   ctx->root = su_root_create(NULL);
   TEST_1(ctx->root);
 
-  /* Disable threading by command line switch? */
   su_root_threading(ctx->root, ctx->threading);
 
   ctx->a.nua = nua_create(ctx->root, a_callback, ctx,
@@ -117,12 +118,12 @@ int test_nua_params(struct context *ctx)
   nua_get_params(ctx->a.nua, TAG_ANY(), TAG_END());
   run_a_until(ctx, nua_r_get_params, save_until_final_response);
 
-  TEST_1(e = ctx->a.events->head);
+  TEST_1(e = ctx->a.specials->head);
   TEST_E(e->data->e_event, nua_r_get_params);
   for (n = 0, t = e->data->e_tags; t; n++, t = tl_next(t))
     ;
   TEST_1(n > 32);
-  free_events_in_list(ctx, ctx->a.events);
+  free_events_in_list(ctx, ctx->a.specials);
 
   nh = nua_handle(ctx->a.nua, NULL, TAG_END()); TEST_1(nh);
   nua_handle_unref(nh);
@@ -148,6 +149,8 @@ int test_nua_params(struct context *ctx)
 
 		 SIPTAG_SUPPORTED_STR("test"),
 		 SIPTAG_ALLOW_STR("DWIM, OPTIONS, INFO"),
+		 NUTAG_APPL_METHOD(NULL),
+		 NUTAG_APPL_METHOD("INVITE, REGISTER, PUBLISH, SUBSCRIBE"),
 		 SIPTAG_ALLOW_EVENTS_STR("reg"),
 		 SIPTAG_USER_AGENT_STR("test_nua/1.0"),
 
@@ -160,6 +163,12 @@ int test_nua_params(struct context *ctx)
 		 NUTAG_INSTANCE("urn:uuid:3eb007b1-6d7f-472e-8b64-29e482795da8"),
 		 NUTAG_OUTBOUND("bar"),
 
+		 NUTAG_INITIAL_ROUTE(NULL),
+		 NUTAG_INITIAL_ROUTE(sip_route_make(tmphome, "<sip:tst@example.net;lr>")),
+		 NUTAG_INITIAL_ROUTE_STR("<sip:str1@example.net;lr>"),
+		 NUTAG_INITIAL_ROUTE_STR("sip:str2@example.net;lr=foo"),
+		 NUTAG_INITIAL_ROUTE_STR(NULL),
+
 		 TAG_END());
 
   run_a_until(ctx, nua_r_set_params, until_final_response);
@@ -190,6 +199,7 @@ int test_nua_params(struct context *ctx)
 		 NUTAG_MEDIA_FEATURES(1),
 		 NUTAG_SERVICE_ROUTE_ENABLE(0),
 		 NUTAG_PATH_ENABLE(0),
+		 NUTAG_AUTH_CACHE(nua_auth_cache_challenged),
 		 NUTAG_REFER_EXPIRES(333),
 		 NUTAG_REFER_WITH_ID(0),
 		 NUTAG_SUBSTATE(nua_substate_pending),
@@ -211,10 +221,19 @@ int test_nua_params(struct context *ctx)
 		 SIPTAG_ALLOW(sip_allow_make(tmphome, "INFO")),
 		 NUTAG_ALLOW("ACK, INFO"),
 
+		 NUTAG_APPL_METHOD("NOTIFY"),
+
 		 SIPTAG_ALLOW_EVENTS_STR("reg"),
-		 SIPTAG_ALLOW_EVENTS(sip_allow_make(tmphome, "presence")),
+		 SIPTAG_ALLOW_EVENTS(sip_allow_events_make(tmphome, "presence")),
 		 NUTAG_ALLOW_EVENTS("presence.winfo"),
 
+		 NUTAG_INITIAL_ROUTE(NULL),
+		 NUTAG_INITIAL_ROUTE(sip_route_make(nua_handle_home(nh), "<sip:1@example.com;lr>")),
+		 NUTAG_INITIAL_ROUTE_STR("<sip:2@example.com;lr>"),
+		 /* Check for sip_route_fix() */
+		 NUTAG_INITIAL_ROUTE_STR("sip:3@example.com;lr=foo"),
+		 NUTAG_INITIAL_ROUTE_STR(NULL),
+
 		 SIPTAG_USER_AGENT(sip_user_agent_make(tmphome, "test_nua")),
 
 		 SIPTAG_ORGANIZATION(sip_organization_make(tmphome, "Pussy Galore's Flying Circus")),
@@ -262,12 +281,14 @@ int test_nua_params(struct context *ctx)
     int media_features = -1;
     int service_route_enable = -1;
     int path_enable = -1;
+    int auth_cache = -1;
     unsigned refer_expires = (unsigned)-1;
     int refer_with_id = -1;
     int substate = -1;
 
     sip_allow_t const *allow = NONE;
     char const *allow_str = "NONE";
+    char const *appl_method = "NONE";
     sip_allow_events_t const *allow_events = NONE;
     char const *allow_events_str = "NONE";
     sip_supported_t const *supported = NONE;
@@ -278,6 +299,9 @@ int test_nua_params(struct context *ctx)
     sip_organization_t const *organization = NONE;
     char const *organization_str = "NONE";
 
+    sip_route_t const *initial_route = NONE;
+    char const *initial_route_str = NONE;
+
     char const *outbound = "NONE";
     char const *m_display = "NONE";
     char const *m_username = "NONE";
@@ -291,7 +315,7 @@ int test_nua_params(struct context *ctx)
     nua_get_params(ctx->a.nua, TAG_ANY(), TAG_END());
     run_a_until(ctx, nua_r_get_params, save_until_final_response);
 
-    TEST_1(e = ctx->a.events->head);
+    TEST_1(e = ctx->a.specials->head);
     TEST_E(e->data->e_event, nua_r_get_params);
 
     n = tl_gets(e->data->e_tags,
@@ -324,6 +348,7 @@ int test_nua_params(struct context *ctx)
 	       	NUTAG_MEDIA_FEATURES_REF(media_features),
 	       	NUTAG_SERVICE_ROUTE_ENABLE_REF(service_route_enable),
 	       	NUTAG_PATH_ENABLE_REF(path_enable),
+	       	NUTAG_AUTH_CACHE_REF(auth_cache),
 	       	NUTAG_REFER_EXPIRES_REF(refer_expires),
 	       	NUTAG_REFER_WITH_ID_REF(refer_with_id),
 	       	NUTAG_SUBSTATE_REF(substate),
@@ -332,6 +357,7 @@ int test_nua_params(struct context *ctx)
 	       	SIPTAG_SUPPORTED_STR_REF(supported_str),
 	       	SIPTAG_ALLOW_REF(allow),
 	       	SIPTAG_ALLOW_STR_REF(allow_str),
+		NUTAG_APPL_METHOD_REF(appl_method),
 		SIPTAG_ALLOW_EVENTS_REF(allow_events),
 		SIPTAG_ALLOW_EVENTS_STR_REF(allow_events_str),
 	       	SIPTAG_USER_AGENT_REF(user_agent),
@@ -341,6 +367,9 @@ int test_nua_params(struct context *ctx)
 	       	SIPTAG_ORGANIZATION_REF(organization),
 	       	SIPTAG_ORGANIZATION_STR_REF(organization_str),
 
+		NUTAG_INITIAL_ROUTE_REF(initial_route),
+		NUTAG_INITIAL_ROUTE_STR_REF(initial_route_str),
+
 	       	NUTAG_REGISTRAR_REF(registrar),
 		NUTAG_KEEPALIVE_REF(keepalive),
 		NUTAG_KEEPALIVE_STREAM_REF(keepalive_stream),
@@ -353,7 +382,7 @@ int test_nua_params(struct context *ctx)
 		NUTAG_INSTANCE_REF(instance),
 
 		TAG_END());
-    TEST(n, 46);
+    TEST(n, 50);
 
     TEST_S(sip_header_as_string(tmphome, (void *)from), Alice);
     TEST_S(from_str, Alice);
@@ -383,12 +412,14 @@ int test_nua_params(struct context *ctx)
     TEST(media_features, 1);
     TEST(service_route_enable, 0);
     TEST(path_enable, 0);
+    TEST(auth_cache, nua_auth_cache_challenged);
     TEST(refer_expires, 333);
     TEST(refer_with_id, 0);
     TEST(substate, nua_substate_pending);
 
     TEST_S(sip_header_as_string(tmphome, (void *)allow), "OPTIONS, INFO, ACK");
     TEST_S(allow_str, "OPTIONS, INFO, ACK");
+    TEST_S(appl_method, "INVITE, REGISTER, PUBLISH, SUBSCRIBE, NOTIFY");
     TEST_S(sip_header_as_string(tmphome, (void *)allow_events), 
 	   "reg, presence, presence.winfo");
     TEST_S(allow_events_str, "reg, presence, presence.winfo");
@@ -401,6 +432,17 @@ int test_nua_params(struct context *ctx)
 	   "Pussy Galore's Flying Circus");
     TEST_S(organization_str, "Pussy Galore's Flying Circus");
 
+    TEST_1(initial_route); TEST_1(initial_route != (void *)-1);
+    TEST_S(initial_route->r_url->url_user, "1");
+    TEST_1(url_has_param(initial_route->r_url, "lr"));
+    TEST_1(initial_route->r_next);
+    TEST_S(initial_route->r_next->r_url->url_user, "2");
+    TEST_1(url_has_param(initial_route->r_next->r_url, "lr"));
+    TEST_1(initial_route->r_next->r_next);
+    TEST_S(initial_route->r_next->r_next->r_url->url_user, "3");
+    TEST_1(url_has_param(initial_route->r_next->r_next->r_url, "lr"));
+    TEST_1(!initial_route->r_next->r_next->r_next);
+
     TEST_S(url_as_string(tmphome, registrar->us_url),
 	   "sip:sip.wonderland.org");
     TEST(keepalive, 66);
@@ -414,7 +456,7 @@ int test_nua_params(struct context *ctx)
     TEST_S(m_features, expect_m_features); }
     TEST_S(outbound, "foo");
 
-    free_events_in_list(ctx, ctx->a.events);
+    free_events_in_list(ctx, ctx->a.specials);
   }
 
   /* Test that only those tags that have been set per handle are returned by nua_get_hparams() */
@@ -446,6 +488,7 @@ int test_nua_params(struct context *ctx)
     int media_features = -1;
     int service_route_enable = -1;
     int path_enable = -1;
+    int auth_cache = -1;
     unsigned refer_expires = (unsigned)-1;
     int refer_with_id = -1;
     int substate = -1;
@@ -459,6 +502,9 @@ int test_nua_params(struct context *ctx)
     sip_organization_t const *organization = NONE;
     char const *organization_str = "NONE";
 
+    sip_route_t *initial_route = NONE;
+    char const *initial_route_str = "NONE";
+
     url_string_t const *registrar = NONE;
 
     char const *outbound = "NONE";
@@ -504,6 +550,7 @@ int test_nua_params(struct context *ctx)
 	       	NUTAG_MEDIA_FEATURES_REF(media_features),
 	       	NUTAG_SERVICE_ROUTE_ENABLE_REF(service_route_enable),
 	       	NUTAG_PATH_ENABLE_REF(path_enable),
+		NUTAG_AUTH_CACHE_REF(auth_cache),
 	       	NUTAG_SUBSTATE_REF(substate),
 
 	       	SIPTAG_SUPPORTED_REF(supported),
@@ -556,6 +603,7 @@ int test_nua_params(struct context *ctx)
     TEST(media_features, -1);
     TEST(service_route_enable, -1);
     TEST(path_enable, -1);
+    TEST(auth_cache, -1);
     TEST(refer_expires, (unsigned)-1);
     TEST(refer_with_id, -1);
     TEST(substate, -1);
@@ -569,6 +617,9 @@ int test_nua_params(struct context *ctx)
     TEST_P(organization, NONE);
     TEST_S(organization_str, "NONE");
 
+    TEST_1(initial_route == (void *)-1);
+    TEST_S(initial_route_str, "NONE");
+
     TEST_S(outbound, "NONE");
     TEST_S(m_display, "NONE");
     TEST_S(m_username, "NONE");
