@@ -55,7 +55,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
+#if HAVE_SIGPIPE
 #include <signal.h>
+#endif
 
 #include "tport_tls.h"
 
@@ -162,8 +165,10 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
     }
   }
 
+#if HAVE_SIGPIPE
   /* Avoid possible SIGPIPE when sending close_notify */
   signal(SIGPIPE, SIG_IGN);
+#endif
 
   if (tls->bio_err == NULL)
     tls->bio_err = BIO_new_fp(stderr, BIO_NOCLOSE);
@@ -295,7 +300,9 @@ tls_t *tls_init_master(tls_issues_t *ti)
   unsigned char sessionId[32] = "sofia/tls"; 
   tls_t *tls;
 
+#if HAVE_SIGPIPE
   signal(SIGPIPE, SIG_IGN);  /* Ignore spurios SIGPIPE from OpenSSL */
+#endif
 
   tls_set_default(ti);
 
@@ -420,6 +427,7 @@ tls_t *tls_clone(tls_t *master, int sock, int accept)
     SSL_set_connect_state(tls->con);
   SSL_set_mode(tls->con, SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
 
+  su_setblocking(sock, 0);
   tls_read(tls); /* XXX - works only with non-blocking sockets */
 
   return tls;
@@ -579,6 +587,10 @@ int tls_error(tls_t *tls, int ret, char const *who, char const *operation,
     return 0;
 
   case SSL_ERROR_SYSCALL:
+    if (SSL_get_shutdown(tls->con) & SSL_RECEIVED_SHUTDOWN)
+      return 0;			/* EOS */
+    if (errno == 0)
+      return 0;			/* EOS */
     return -1;
 
   default:
@@ -611,7 +623,7 @@ ssize_t tls_read(tls_t *tls)
   }
 
   if (0)
-    fprintf(stderr, "tls_read(%p) called on %s (events %u)\n", tls,
+    fprintf(stderr, "tls_read(%p) called on %s (events %u)\n", (void *)tls,
 	    tls->type == tls_slave ? "server" : "client",
 	    tls->read_events);
 
@@ -652,20 +664,30 @@ int tls_pending(tls_t const *tls)
   return tls && tls->con && SSL_pending(tls->con);
 }
 
+/** Check if data is available in TCP connection.
+ *
+ * 
+ *
+ * @retval -1 upon an error
+ * @retval 0 end-of-stream
+ * @retval 1 nothing to read 
+ * @retval 2 there is data to read
+ */
 int tls_want_read(tls_t *tls, int events)
 {
   if (tls && (events & tls->read_events)) {
     int ret = tls_read(tls);
-
-    if (ret >= 0)
-      return 1;
-    else if (errno == EAGAIN)
+    if (ret > 0)
+      return 2;
+    else if (ret == 0)
       return 0;
+    else if (errno == EAGAIN)
+      return 3;			/* ??? */
     else
       return -1;
   }
 
-  return 0;
+  return 1;
 }
 
 ssize_t tls_write(tls_t *tls, void *buf, size_t size)
@@ -674,7 +696,7 @@ ssize_t tls_write(tls_t *tls, void *buf, size_t size)
 
   if (0) 
     fprintf(stderr, "tls_write(%p, %p, "MOD_ZU") called on %s\n", 
-	    tls, buf, size,
+	    (void *)tls, buf, size,
 	    tls && tls->type == tls_slave ? "server" : "client");
 
   if (tls == NULL || buf == NULL) {
